function figure2()
% This script generates the two parts of Figure 2 from Naik et al
%
% The data used here were generated from the active learner in 022011_prime

% This file is automatically generated by the active learner (022011_prime
% version). Each column indicates (1/0) whether a particular method was run
% so for example, the active learner cases that used the Hierarchical
% Dirichlet Process as the structure learning model have a '1' in the 
% column labeled "HDP". We also indicate which merge ordering was used for
% the imputations, which imputations were used (e.g. 'COLOR_3PT') and so
% on. Some of the methods (in particular, HDP) were not mentioned by name 
% in the paper because the results were very poor.
% 

% Copyright (C) 2013,2014 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

METHOD_ARRANGEMENT = importdata('CASES_FOR_TABLE_1_FOR_MATLAB.csv');

COLUMN_OF_NONSTRICT_METHOD = find(not(cellfun('isempty', strfind(METHOD_ARRANGEMENT.colheaders, 'BOGUS_nonstrict'))));
COLUMN_OF_STRICT_METHOD = find(not(cellfun('isempty', strfind(METHOD_ARRANGEMENT.colheaders, 'BOGUS_strict'))));

% This is the design matrix; each method (indexed from 1) is a row in this
% matrix and corresponds to a combination of clustering, modeling
% and imputation methods. This is used to pull out results from many
% different simulations
design = METHOD_ARRANGEMENT.data;


% "rawdata" consists of entries describing each point of many simulations
% its first column is a method code, which indicates what combination of 
% clustering and modeling methods was used for the simulation that it
% is from). These data are produced by simulations which were instantiated
% by calling make_figure2_data.pl in the parent directory. Each of those
% simulations (which are encapsulated in separate directories) have a
% subdirectory called "results"; the csv files in these are concatenated to
% produce an aggregate data file called "figure2_data.csv" which is used
% here

rawdata = csvread('figure2_data.csv');
data = rawdata(:,2:end);

%
% after offsetting to remove the method, the columns in "data" are
% 1 number of targets 
% 2 number of conditions 
% 3 number of phenotypes 
% 4 responsive 
% 5 replicated 
% 6 active/random (1=active, 2=random)
% 7 sample size 
% 8 performance 
% 9 #clusters 
% 10 ovpairs
% 11 # phenotypes
% 12 #twins 
% 13 previous model agreement 
% 14 color eps-net
% 15 equiv eps-net
% 16 median color eps-approx
% 17 median equiv eps-approx
% 18 max color eps-approx
% 19 max equiv eps-approx
% 20 in-model responders
% 21 observed responders

methods = unique(rawdata(:,1));

% For each method we average over the random seeds and random data samples
% per batch. These are stored in a very large cell array indexed by method

results = cell(size(methods,1),1);

for i = 1:size(methods,1)
    aplaces = find((methods(i) == rawdata(:,1)) & data(:,6)==1);
    rplaces = find((methods(i) == rawdata(:,1)) & data(:,6)==2);

    performance = zeros(100,1);
    prevagree = zeros(100,1);
    colornet = zeros(100,1);
    equivnet = zeros(100,1);
    inresp = zeros(100,1);
    obsresp = zeros(100,1);
    
    theplaces = aplaces;
    for j = 1:100
        performance(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),8));
        prevagree(j) = mean(data(theplaces(find(data(theplaces,13)==100*j)),8));
        colornet(j) = mean(data(theplaces(find(data(theplaces,14)==100*j)),8));
        equivnet(j) = mean(data(theplaces(find(data(theplaces,15)==100*j)),8));
        inresp(j) = mean(data(theplaces(find(data(theplaces,20)==100*j)),8));
        obsresp(j) = mean(data(theplaces(find(data(theplaces,21)==100*j)),8));
    end
    results{i}.active = struct('performance', performance, 'prevagree', prevagree, 'colornet', colornet, 'equivnet', equivnet, 'inresp', inresp, 'obsresp', obsresp);
    
    theplaces = rplaces;
    for j = 1:100
        performance(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),8));
        prevagree(j) = mean(data(theplaces(find(data(theplaces,13)==100*j)),8));
        colornet(j) = mean(data(theplaces(find(data(theplaces,14)==100*j)),8));
        equivnet(j) = mean(data(theplaces(find(data(theplaces,15)==100*j)),8));
        inresp(j) = mean(data(theplaces(find(data(theplaces,20)==100*j)),8));
        obsresp(j) = mean(data(theplaces(find(data(theplaces,21)==100*j)),8));
    end
    results{i}.random = struct('performance', performance, 'prevagree', prevagree, 'colornet', colornet, 'equivnet', equivnet, 'inresp', inresp, 'obsresp', obsresp);
end

% normalize the data (we know that the matrix was 100x100)
aperformances = zeros(size(methods,1),100);
rperformances = zeros(size(methods,1),100);
for i = 1:size(methods,1)
    aperformances(i,:) = results{i}.active.performance/10000;
    rperformances(i,:) = results{i}.random.performance/10000;
end

% the interesting subset that exhibited significant learning improvement
% over a model free learner. This cutoff (>=70% accurate at the 50% mark)
% was determined by inspection
mms = find(aperformances(:,50)>.7);
figure; hold on;
for i = 1:size(methods,1)
    if isempty(find(mms==i))
        c = 'k';
        above_symb = '';
    else
        above_symb = 'o';
        if (design(i,COLUMN_OF_STRICT_METHOD)==1)
            c = 'r';
        elseif (design(i,COLUMN_OF_NONSTRICT_METHOD)==1)
            c = 'b';
        else
            % B-Clustering case 
            c = 'g';
        end
    end
    for j = 2:100
        if ((aperformances(i,j)-rperformances(i,j))>0)
            plot([j-1,j],[aperformances(i,j-1) aperformances(i,j)], [c above_symb '-'], 'LineWidth', 1, 'MarkerFaceColor', c, 'MarkerSize',4);
        else
            plot([j-1,j],[aperformances(i,j-1) aperformances(i,j)], [c '-'], 'LineWidth', 1);
        end
    end
end
hXLabel = xlabel('Batch');
hYLabel = ylabel('Accuracy');
set([hXLabel, hYLabel], ...
    'FontName'   , 'Helvetica', ...
    'FontSize'   , 25);
set([hXLabel, hYLabel], ...
    'FontWeight' , 'bold'      );
set(gcf, 'PaperPositionMode', 'auto');
set(gca, ...
  'Box'         , 'off'     , ...
  'TickDir'     , 'out'     , ...
  'TickLength'  , [.02 .02] , ...
  'XColor'      , [.0 .0 .0], ...
  'YColor'      , [.0 .0 .0], ...
  'XMinorTick'  , 'on', ...
  'YMinorTick'  , 'on', ...
  'FontSize', 25, ...
  'LineWidth'   , 1         );
print(gcf, '-dpng', 'figure2a.png', '-r300');

%%
figure; hold on;
for z = 1:size(mms,1)
    i = mms(z);
    if (design(i,COLUMN_OF_STRICT_METHOD)==1)
        tok = 'r';
    elseif (design(i,COLUMN_OF_NONSTRICT_METHOD)==1)
        tok = 'b';
    else
        % B-Clustering case 
        tok = 'g';
    end
    z = min(find(max(aperformances(i,:),rperformances(i,:))==1));
    plot(1:z, aperformances(i,1:z)-rperformances(i,1:z), [tok '-'], 'LineWidth', 1)
    plot(z:100, aperformances(i,z:100)-rperformances(i,z:100), [tok 'o-'], 'LineWidth', 1, 'MarkerFaceColor', tok, 'MarkerSize',4)
end
hXLabel = xlabel('Batch');
hYLabel = ylabel('Active - Random Learner Accuracy');
set([hXLabel, hYLabel], ...
    'FontName'   , 'Helvetica', ...
    'FontSize'   , 25);
set([hXLabel, hYLabel], ...
    'FontWeight' , 'bold'      );
set(gcf, 'PaperPositionMode', 'auto');
set(gca, ...
  'Box'         , 'off'     , ...
  'TickDir'     , 'out'     , ...
  'TickLength'  , [.02 .02] , ...
  'XColor'      , [.0 .0 .0], ...
  'YColor'      , [.0 .0 .0], ...
  'XMinorTick'  , 'on', ...
  'YMinorTick'  , 'on', ...
  'FontSize', 25, ...
  'LineWidth'   , 1         );
axis([0 100 -0.12 0.12])
print(gcf, '-dpng', 'figure2b.png', '-r300');