function figure3and4()
% this script generates the pieces of Figures 3 and 4 from Naik et al
% these data were produced by simulations made by calling
% make_stopping_data.py from the parent directory

% the simulation data were aggregated into b-cluster only runs
% and greedy merge variants separately, but this isn't required
% since these files are just concatenations of the relevant individual runs
% these files

% Copyright (C) 2013,2014 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

rawdata1 = csvread('final_32pheno.csv');
rawdata2 = csvread('bcluster_32.csv');
rawdata = [rawdata1; rawdata2];

% see figure2.m for definition of columns in rawdata
data = rawdata(:,2:end);

points = unique(data(:,4:5), 'rows');
npoints = size(points,1);
arow = unique(points(:,1));
acol = unique(points(:,2));
rev_points = zeros(size(points));
for i = 1:size(points,1)
   rev_points(i,1) = find(arow == points(i,1));
   rev_points(i,2) = find(acol == points(i,2));
end

methods = unique(rawdata(:,1));

% for each method...

results = cell(size(methods,1),1);

% similar code in figure2.m
for i = 1:size(methods,1)
    here = cell(size(points,1),1);
    for z = 1:size(points,1)
        aplaces = find((methods(i) == rawdata(:,1)) & (data(:,6)==1) & (data(:,4)==points(z,1)) & (data(:,5)==points(z,2)));
        rplaces = find((methods(i) == rawdata(:,1)) & (data(:,6)==2) & (data(:,4)==points(z,1)) & (data(:,5)==points(z,2)));

        performance = zeros(100,1);
        sperf = zeros(100,1);
        prevagree = zeros(100,1);
        colornet = zeros(100,1);
        equivnet = zeros(100,1);
        inresp = zeros(100,1);
        obsresp = zeros(100,1);
        nclust = zeros(100,1);
    
        theplaces = aplaces;
        for j = 1:100
            performance(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),8));
            sperf(j) = std(data(theplaces(find(data(theplaces,7)==100*j)),8));
            prevagree(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),13));
            colornet(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),14));
            equivnet(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),15));
            inresp(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),20));
            obsresp(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),21));
            nclust(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),9));
        end
        here{z}.active = struct('performance', performance, 'sperf', sperf, 'prevagree', prevagree, 'colornet', colornet, 'equivnet', equivnet, 'inresp', inresp, 'obsresp', obsresp, 'nclust', nclust);
    
        theplaces = rplaces;
        for j = 1:100
            performance(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),8));
            sperf(j) = std(data(theplaces(find(data(theplaces,7)==100*j)),8));
            prevagree(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),13));
            colornet(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),14));
            equivnet(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),15));
            inresp(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),20));
            obsresp(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),21));
            nclust(j) = mean(data(theplaces(find(data(theplaces,7)==100*j)),9));
        end
        here{z}.random = struct('performance', performance, 'sperf', sperf, 'prevagree', prevagree, 'colornet', colornet, 'equivnet', equivnet, 'inresp', inresp, 'obsresp', obsresp, 'nclust', nclust);
    end
    results{i} = here;
end

%% difference in #batches to 100%
level = 10000;
actual1 = zeros(20,20);
actual2 = zeros(20,20);
actual3 = zeros(20,20);
differences1 = zeros(20,20);
differences2 = zeros(20,20);
differences3 = zeros(20,20);
%this section is for the "strict" version of Greedy Merge, which was run
%but not used in the paper since its performance is better
for i = 1:size(points,1)
    here = results{1}{i};
    a = min(find(here.active.performance>=level));
    if (isempty(a))
        a = 100;
    end
    b =  min(find(here.random.performance>=level));
    if (isempty(b))
        b = 100;
    end
    differences1(rev_points(i,1), rev_points(i,2)) = b-a;
    actual1(rev_points(i,1), rev_points(i,2)) = a;
end
%this is for Greedy Merge (both Merge Overlapping and Merge Nonconflicting)
for i = 1:size(points,1)
    here = results{2}{i};
    a = min(find(here.active.performance>=level));
    if (isempty(a))
        a = 100;
    end
    b =  min(find(here.random.performance>=level));
    if (isempty(b))
        b = 100;
    end
    differences2(rev_points(i,1), rev_points(i,2)) = b-a;
    actual2(rev_points(i,1), rev_points(i,2)) = a;
end
%this is for B-clustering
for i = 1:size(points,1)
    here = results{3}{i};
    a = min(find(here.active.performance>=level));
    if (isempty(a))
        a = 100;
    end
    b =  min(find(here.random.performance>=level));
    if (isempty(b))
        b = 100;
    end
    differences3(rev_points(i,1), rev_points(i,2)) = b-a;
    actual3(rev_points(i,1), rev_points(i,2)) = a;
end

mm = max([max(abs(differences1(:))) max(abs(differences2(:))) max(abs(differences3(:)))]);
%myhmap2(differences1,mm);
%print(gcf, '-dtiff', 'figure3_strict_32pheno_100percent.tiff', '-r300');
%clf
myhmap2(differences2,mm);
print(gcf, '-dtiff', 'figure3_nostrict_32pheno_100percent.tiff', '-r300');
clf
myhmap2(differences3,mm);
print(gcf, '-dtiff', 'figure3_bcluster_32pheno_100percent.tiff', '-r300');

%% difference in #batches to 90%
level = 9000;
actual1 = zeros(20,20);
actual2 = zeros(20,20);
actual3 = zeros(20,20);
differences1 = zeros(20,20);
differences2 = zeros(20,20);
differences3 = zeros(20,20);
for i = 1:size(points,1)
    here = results{1}{i};
    a = min(find(here.active.performance>=level));
    if (isempty(a))
        a = 100;
    end
    b =  min(find(here.random.performance>=level));
    if (isempty(b))
        b = 100;
    end
    differences1(rev_points(i,1), rev_points(i,2)) = b-a;
    actual1(rev_points(i,1), rev_points(i,2)) = a;
end
for i = 1:size(points,1)
    here = results{2}{i};
    a = min(find(here.active.performance>=level));
    if (isempty(a))
        a = 100;
    end
    b =  min(find(here.random.performance>=level));
    if (isempty(b))
        b = 100;
    end
    differences2(rev_points(i,1), rev_points(i,2)) = b-a;
    actual2(rev_points(i,1), rev_points(i,2)) = a;
end
for i = 1:size(points,1)
    here = results{3}{i};
    a = min(find(here.active.performance>=level));
    if (isempty(a))
        a = 100;
    end
    b =  min(find(here.random.performance>=level));
    if (isempty(b))
        b = 100;
    end
    differences3(rev_points(i,1), rev_points(i,2)) = b-a;
    actual3(rev_points(i,1), rev_points(i,2)) = a;
end
mm = max([max(abs(differences1(:))) max(abs(differences2(:))) max(abs(differences3(:)))]);
%myhmap2(differences1,mm);
%print(gcf, '-dtiff', 'figure3_strict_32pheno_90percent.tiff', '-r300');
%clf
myhmap2(differences2,mm);
print(gcf, '-dtiff', 'figure3_nostrict_32pheno_90percent.tiff', '-r300');
clf
myhmap2(differences3,mm);
print(gcf, '-dtiff', 'figure3_bcluster_32pheno_90percent.tiff', '-r300');

%% plot learning curves for various uniqueness and responsiveness values

scurve(results, points, 10, 70);
scurve(results, points, 90, 25);


