from okasaki import okasaki
import networkx
import string


# ==============================================================================
# base class: task data
# ==============================================================================

class task_data:
    def __init__(self):
        self.graph = networkx.DiGraph()
        self.job_data = {}
        self.frontier = okasaki()
        self.to_submit = okasaki()
        self.name = ''

    def update(self, jobs_finished):

        somethinghappened = False
        # for each job finished, mark the successors as the frontier of what might be queued (IN ORDER)
        for j in jobs_finished:
            for j_new in self.graph.neighbors(j):
                if not ( self.frontier.inside(j_new) ):
                    self.frontier.enq(j_new)
                    somethinghappened = True
            self.graph.remove_node(j)
            
        # invariant: jobs that are ready to be run have no predecessors

        # if a frontier job is ready to be run, add it to the to_submit queue
        remacc = []
        for j in self.frontier.walk():
            if ( ( not self.graph.predecessors(j) ) and not ( self.to_submit.inside(j) ) ):
                self.to_submit.enq(j)
                somethinghappened = True
                remacc.append(j)

        for j in remacc:
            self.frontier.remove(j)

        if somethinghappened:
            print 'UPDATE '+self.name+' frontier =', self.frontier.len(), 'to_submit =', self.to_submit.len()


    def is_done(self):
        return ( self.frontier.len() == 0 ) and ( self.to_submit.len() == 0 ) and ( len(self.graph.nodes()) == 0 )


# ==============================================================================
# job thread data
# ==============================================================================

class job_thread_data(task_data):
    def __init__(self):
        task_data.__init__(self)

    def load_from_file(self, fname):
        print "loading thread: "+fname
        self.name = fname

        f = open(fname, 'r')
        infile = f.read()

        tokens = string.split(infile, "\n")[:-1]

        for line in tokens:
            t = line.split()
            # format: job_name runlength memory [dependencies]
            self.job_data[t[0]] = (t[0], t[1], t[2])

            self.graph.add_node(t[0])

            if ( len(t) == 3 ):
                self.frontier.enq(t[0])
            else:
                for j_parent in t[3:]:
                    self.graph.add_edge(j_parent, t[0])
#            print "NEW JOB:", self.name, t[0]


# ==============================================================================
# job bundle data
# ==============================================================================

class job_bundle_data(task_data):
    def __init__(self):
        task_data.__init__(self)

    def add_task(self, taskf, deps):

        print "adding task: "+taskf

        self.graph.add_node(taskf)
        self.job_data[taskf] = taskf

        if (len(deps) == 0):
            self.frontier.enq(taskf)
        else:
            for tparent in deps:
                self.graph.add_edge(tparent, taskf)
