import numpy
import os
import sys
import string

# we reconstruct the data based on the active learning simulations, which have
# outside of this script been copied to the local directory
places = [x for x in os.listdir('.') if 'SESSION' in x]

runs = [x for x in os.listdir('..') if ( 'lincs_' in x ) and os.path.isdir(x)]

if len(runs)==0:
    print '...assuming that either you are trying to recreate figures from saved intermediate data'
    print 'or that you have not run the simulations yet (make_lincs_data.py in parent directory)'
    sys.exit(0)

print '...creating summary data from simulation results (n='+str(len(runs))+')'

# this is keyed on the number of phenotypes for the simulation (active and random pairs)
paired = {}
for x in places:
    line = string.split(x[:-4],'__')
    if len(line) == 6:
        if int(line[4]) not in paired:
            # active and random slots
            paired[int(line[4])] = ([], [])
        fin = open(x,'r')
        for rline in fin:
            Line = string.split(rline.rstrip(),',')
            batch = int(Line[6])
            accuracy = int(Line[7])
            nclusters = int(Line[10])
            ntwins = int(Line[11])
            paired[int(line[4])][int(line[5])-1].append([batch, accuracy, nclusters, ntwins])
        fin.close()

# just keep the ones that are full length
ok = []
for p in paired.keys():
    if ( len(paired[p][0]) == 279 ) and ( len(paired[p][1]) == 279 ):
        ok.append(p)

# compute the differences in active vs random number of batches necessary to attain 95% accuracy
overing = []
for p in ok:
    paired[p] = (numpy.asarray(paired[p][0]), numpy.asarray(paired[p][1]))
    overing.append(numpy.min(numpy.where(paired[p][1][:,1]>=0.95*280*50)[0])-numpy.min(numpy.where(paired[p][0][:,1]>=0.95*280*50)[0]))

# compute the uniqueness and responsiveness parameters for each experiment matrix by parsing one of the simulation input files
params = []
for k in ok:
    z = numpy.loadtxt('../lincs_sd1_ph'+str(k)+'_b50/this_case.csv',delimiter=',')
    unique_rows = numpy.array([numpy.array(x) for x in set(tuple(x) for x in z)])
    unique_cols = numpy.array([numpy.array(x) for x in set(tuple(x) for x in unique_rows.T)]).T
    responders = numpy.where(unique_cols != numpy.tile(unique_cols[:,0,numpy.newaxis],unique_cols.shape[1]))[0].shape[0]
    params.append((float(numpy.prod(unique_cols.shape))/numpy.prod(z.shape), float(responders)/numpy.prod(unique_cols.shape)))

params = numpy.asarray(params)

numpy.savetxt('params.csv', params, delimiter=',')
numpy.savetxt('overing.csv', overing, delimiter=',')
