function figure5()
% this script generates the pieces of Figure 5 from Naik et al
% these data were produced by simulations made by calling
% make_stopping_data.py from the parent directory

% Copyright (C) 2013,2014 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% concatenation of active*.csv files
actived = importdata('ACTIVE.csv');
%actived(isnan(actived)) = 0;
actived = actived(~sum(isnan(actived),2),:);

% these are the columns of the data; 
% 01 WHICH_SEED_TO_USE,
% 02 DATA_SAMPLE,
% 03 THE_BATCH_SIZE,
% 04 NOISE_LEVEL,
% 05 NUM_DRUGS, 
% 06 NUM_PROTEINS, 
% 07 NUM_UNIQUE_PHENOTYPES,
% 08 FRACTION_RESPONSIVE,
% 09 FRACTION_UNIQUE,
% 10 WHICH_METHOD,
% 11 WHICH_LEARNER
% 12 batch
% 13 true performane
% 14 comparision performance
% 15 delta performance
% 16 clusters
% 17 phenotypes
% 18 golden min color edge hit
% 19 golden max color edge hit
% 20 golden max eps approx
% 21 golden average color population
% 22 comparision min color edge hit
% 23 comparision max color edge hit
% 24 comparision max eps approx
% 25 comparision average color population
% 26 delta min color edge hit
% 27 delta max color edge hit
% 28 delta max eps approx
% 29 delta average color population
% = 30-32 are computed totally incorrectly!!!!
% 30 raw responsiveness
% 31 compressed responsiveness
% 32 frontier responsiveness
% 33 inner responsiveness
% 34 oi responsiveness
% 35 golden color match quality
% 36 golden equivalence class match quality
% 37 comparision color match quality
% 38 comparision equivalence class match quality
% 39 delta color match quality
% 40 delta equivalence class match quality

% normalize to fraction
target_active = [actived(:,13)/(100*100)];
orig_target_active = target_active;
% subtract out expected contribution from (linear) random learning
target_active = target_active - actived(:,12)/100;

% these are the subset that are known to a practitioner (i.e. can be
% computed from observations alone
useful_positions = [12,15,16,17,26,27,28,29,33,34,39,40];
ppx = nchoosek(1:length(useful_positions),2);
design_active = zeros(size(actived,1),length(useful_positions) + size(ppx,1));
design_active(:,1:length(useful_positions)) = zscore(actived(:,useful_positions));
design_active(:,1:length(useful_positions)) = design_active(:,1:length(useful_positions));

% form pairwise products of the selected features as well
for i = 1:length(ppx)
    design_active(:,i+length(useful_positions)) = design_active(:,ppx(i,1)).*design_active(:,ppx(i,2));
end
design_active = zscore(design_active);

path([pwd '/../matlab/glmnet_matlab'], path)

special_target = log(target_active./(1 - target_active));

% cross validate to find a good LASSO regression penalty
ntrials = 10
paramm = glmnetSet;
pp = zeros(1,size(target_active,2));
n = size(target_active,1)
nsub = floor(n/ntrials)
tests = zeros(ntrials,nsub);
fits = {}
for j = 1:ntrials
    j
    myperm = randperm(n);   
    train = sort(myperm(1:(n-nsub)));
    tests(j,:) = sort(myperm((n-nsub+1):n));  
    fits{j} = glmnet(design_active(train,:), special_target(train,1), 'gaussian');
end

minlambda = inf
maxlambda = -inf
for j = 1:ntrials
    minlambda = min(minlambda, min(fits{j}.lambda(~isnan(fits{j}.dev))));
    maxlambda = max(maxlambda, max(fits{j}.lambda(~isnan(fits{j}.dev))));
end
maxlambda = maxlambda/2;

% choose the penalty that leads to an estimator that is a good lower bound
% of the true accuracy (as opposed to the least squared error). We're doing
% this with a LASSO estimator rather than use a general purpose solver for
% simplicity
nlambdas = 100
fitty = zeros(ntrials, nlambdas);
tlambdas = (minlambda:((maxlambda-minlambda)/nlambdas):maxlambda)'
tlambdas = tlambdas(1:nlambdas);
for j = 1:ntrials
    predicts = glmnetPredict(fits{j}, 'link',design_active(tests(j,:),:), tlambdas);
    epredicts = 1.0./(exp(-1.0*predicts)+1.0);   
    fitty(j,:) = sum(repmat(target_active(tests(j,:)), 1, nlambdas) < epredicts,1);
end

[a,b] = min(sum(abs(fitty),1))

% cross validated lambda
thelambda = tlambdas(b);

% compute the fit with the whole data, using the cross validated lambda
fit = glmnet(design_active, special_target);
raw_pred_active = glmnetPredict(fit, 'link',design_active, thelambda);
% logistic link
pred_active = 1.0./(exp(-1.0*raw_pred_active)+1.0);

glmbeta = glmnetCoef(fit, thelambda);
glmbeta = glmbeta(2:end);
subdesign = design_active(:,glmbeta~=0);
% we have to use lscov for numerical stability reasons
raw_pred_beta = lscov(subdesign, special_target) %(subdesign'*subdesign)^-1*subdesign'*special_target;
OLR_raw_pred_active = subdesign*raw_pred_beta;
% logistic link
OLR_pred_active = 1.0./(exp(-1.0*OLR_raw_pred_active)+1.0);



%%
offset_target = orig_target_active;
prefix = 'OLS_';

% add back in the contribution from sampling
if strcmp(prefix,'OLS_')==1
    'OLS!!'
    offset_active = OLR_pred_active+actived(:,12)/100;
else
    offset_active = pred_active+actived(:,12)/100;
end

offset_active = offset_active/max(offset_active);
found = (offset_target ) >= (offset_active );

tpred_active = round(offset_active*100)/100;
ttarget_active = round(offset_target*100)/100;
maxx_place_raw = round(100*max(offset_active))
maxx_place = (maxx_place_raw/100)*1.1;
places = ((0:maxx_place_raw)/100)';

perf = zeros(length(places),1);
found = offset_target >= offset_active;
for i = 1:size(places,1)
    hitting = tpred_active == places(i);
    z = sum(hitting);
    perf(i) = sum(hitting & found)/z;
end
%subplot(3,1,1)
figure;
plot(places, perf, 'k-', 'LineWidth', 2)
xlabel('Predicted Accuracy Score')
ylabel('Probability that True Accuracy Meets or Exceeds Prediction')
axis([0 maxx_place -0.1 1.1])
set(gca, 'XTick', 0:.1:maxx_place)
set(gca, 'YTick', 0:.1:1.1)
print(gcf, '-dpng', [prefix 'figure5_probably_approximately_correct.png'], '-r300');


kk = 300000;


preddens = zeros(101, length(places));
preddens(:) = NaN;
for j = 1:100
    atbatch = actived(:,12)==j;
    for i = 1:length(places)
        vibrations = (tpred_active==places(i));
        preddens(j,i) = sum(found & vibrations & atbatch)/sum(vibrations & atbatch);
    end
end
preddens(isinf(preddens)) = NaN;

line_preddens = preddens(sub2ind(size(preddens), actived(:,12), floor(tpred_active*100)));
ok = ~isnan(line_preddens);
subsubsample = find(ok);
subsample = randperm(length(subsubsample));
pxx = subsubsample(subsample(1:kk));
figure;
scatter(offset_active(pxx), actived(pxx,12), 5, line_preddens(pxx), 'filled')
colormap copper
xlabel('Predicted Accuracy Score')
ylabel('Batch Number')
axis([0 maxx_place 0 100])
set(gca, 'XTick', 0:.1:maxx_place)
print(gcf, '-dpng', [prefix 'figure5_readout_probably_approximately_correct_no_colorbar.png'], '-r300');
h = colorbar;
ylabel(h, 'Probability that True Accuracy Meets or Exceeds Prediction')
print(gcf, '-dpng', [prefix 'figure5_readout_probably_approximately_correct.png'], '-r300');


density = zeros(length(places));
incrs = sub2ind(size(density), floor(tpred_active*100)+1, floor(ttarget_active*100)+1);
for i = 1:length(incrs)
    density(incrs(i)) = density(incrs(i))+1;
end
density = density/max(density(:));
%density(density==0) = NaN;
line_density = density(sub2ind(size(density), floor(tpred_active*100), floor(ttarget_active*100)));
ok = ~isnan(line_density);
subsubsample = find(ok);
subsample = randperm(length(subsubsample));
pxx = subsubsample(subsample(1:kk));

figure;
scatter(offset_active(pxx), offset_target(pxx), 1, line_density(pxx));
colormap(copper(1000))
xlabel('Predicted Accuracy Score')
ylabel('True Accuracy')
axis([0 maxx_place 0 1])
set(gca, 'XTick', 0:.1:maxx_place)
set(gca, 'YTick', 0:.1:1)
print(gcf, '-dpng', [prefix 'figure5_marginal_density_of_predictions_no_colorbar.png'], '-r300');
h = colorbar;
ylabel(h, 'Marginal Density')
print(gcf, '-dpng', [prefix 'figure5_marginal_density_of_predictions.png'], '-r300');

