from okasaki import okasaki
from job_thread_data import job_thread_data
from job_thread_data import job_bundle_data

import networkx
import os
import time
import subprocess
#import sets

class thread_manager:
    def __init__(self, pollname):
        self.pollname = pollname
        self.last_time_stamp = time.time()
        self.submission_budget = 72
        self.jthreads = {}
        self.jobs_live = {}
        self.thread_ordering = []
        self.jobs_finished = {}
        self.thread_bundle = job_bundle_data()
        self.thread_bundle.name = 'thread bundle'
        
    # ---------------------------------------------------------------------------
    # Poll Jobs
    # ---------------------------------------------------------------------------

    def poll_queue_and_update_status(self, forceprint):
        try:
            output = subprocess.check_output(["qstat"])
            outputn = output.split()
            outputn = outputn[14:]  # removing unnecessary headers from qstat output
            alive = []
            waiting = 0
            running = 0
            for i in xrange(0, len(outputn), 6):  # jobids are every sixth index in outputn
                id = outputn[i].split('.',1)  # splitting into ['#', 'lanec1']
                if self.jobs_live.has_key(id[0]):
                    alive.append(id[0])
                    if outputn[i+4] == "R":
                        running += 1
                    else:
                        waiting += 1

            finished = 0
            for qj in list(set(self.jobs_live.keys()) - set(alive)):
                finished += 1
                j, t = self.jobs_live[qj]
                # invariant: jobs_finished has been allocated with at least an empty list
                self.jobs_finished[t].append(j)
                del self.jobs_live[qj]

            if ( finished > 0 ) or forceprint:
                print "POOL STATUS running =", running, "waiting =", waiting, "finished =", finished
        except:
            return
                
    # ---------------------------------------------------------------------------
    # Submit Jobs
    # ---------------------------------------------------------------------------

    def submit_jobs(self):
        nthreads = len(self.jthreads)
        
        num_jobs_can_submit = 0

        enabled_threads = []
        for t in self.thread_ordering:
            can_submit = self.jthreads[t].to_submit.len()
            if can_submit > 0:
                enabled_threads.append(t)
                num_jobs_can_submit += can_submit

        nenabled = len(enabled_threads)
        
        budget = min([self.submission_budget - len(self.jobs_live), num_jobs_can_submit])

        if budget == 0:
            return

        # required for fairness - was a bug
        if budget < nenabled:
            enabled_threads = enabled_threads[0:budget]
            nenabled = budget

        print 'SUBMITTING', budget, 'out of', num_jobs_can_submit, 'from', nenabled, 'threads'

        curthread = 0
        for b in xrange(0, budget):
            t = enabled_threads[curthread]
            if ( self.jthreads[t].to_submit.len() > 0 ):
                #time.sleep(10.0)
                time.sleep(1.0)
                job = self.jthreads[t].to_submit.peek()
                command, walltime, memsize = self.jthreads[t].job_data[job]
                lcmd= "walltime="+walltime+",mem="+memsize
                try:
                    output = subprocess.check_output(["qsub", "-q", "pool1", "-l", lcmd, command])
                    zz = output.split('.',1)
                    self.jobs_live[zz[0]] = (job,t)
                    self.jthreads[t].to_submit.deq()
                except:
                    if b < nenabled:
                        # make sure we preserve relative thread ordering
                        # if we barf in the first pass through, we know that only threads 0..(b-1) actually submitted a job
                        # therefore, we need to discard all 'enabled' threads from b..enabled inclusive so that
                        # they are replaced in the same (old) thread ordering 
                        enabled_threads = [enabled[x] for x in xrange(0,(b-1))]
                    print 'WARNING *** qsub command defered - only submitted', b, 'out of', budget
                    break
                
            curthread = ( curthread + 1 ) % nenabled

        # update the thread ordering
        # carefully maintain the relative priority of those threads that
        # did not run previously, and might not have run again
        prefix_ord = []
        suffix_ord = []
        for t in self.thread_ordering:
            if t in enabled_threads:
                suffix_ord.append(t)
            else:
                prefix_ord.append(t)
        self.thread_ordering = prefix_ord + suffix_ord

    # ---------------------------------------------------------------------------
    # Poll for Control
    # ---------------------------------------------------------------------------

    def poll_for_control(self):
#        print "poll_for_control:  starting"
        try:
            new_time_stamp = os.path.getmtime(self.pollname)
        except:
            print "poll_for_control:  could not open pollfile"
            return
        
        if ( new_time_stamp != self.last_time_stamp ):
            print "poll_for_control:  new_time_stamp is "+str(new_time_stamp)
            self.last_time_stamp = new_time_stamp
            f = open(self.pollname, 'r')
#            print "updating..."
            for infile in f:
                tokens = infile.split()

                # load a new thread
                if ( tokens[0] == "ADDTHREAD" ):  # tokens[1] must then be a path
                    if ( len(tokens) > 2 ):
                        self.thread_bundle.add_task(tokens[1], tokens[2:])
                    else:
                        self.thread_bundle.add_task(tokens[1], [])
                        
                elif ( tokens[0] == "BUDGET" ):  # tokens[1] must then be a number
                    self.submission_budget = int(tokens[1])
                    print 'poll_for_control: new_submission_budget is now '+str(self.submission_budget)
                else:
                    print 'poll_for_control:  tokens[0] is not ADDTHREAD or BUDGET -->  INPUT ERROR'
            f.close()

    # ---------------------------------------------------------------------------
    # Inner Thunk
    # ---------------------------------------------------------------------------

    def inner_thunk(self, innercount):
#        print "inner_thunk:  going..."

        self.poll_for_control()

        # are any threads done? if so, remove them
        rmacc = []
        for t in self.jthreads:
            if self.jthreads[t].is_done():
                rmacc.append(t)

        for t in rmacc:
            del self.jthreads[t]
            del self.jobs_finished[t]
            self.thread_ordering.remove(t)

        # update the thread bundle
        self.thread_bundle.update(rmacc)

        # if there are new threads that came from polling or are now enabled, add them
        new_thread_ordering = []
        for x in range(0,self.thread_bundle.to_submit.len()):
#            print "THREAD WAKEUP: ", x
            tn = self.thread_bundle.to_submit.deq()
            self.jthreads[tn] = job_thread_data()
            self.jthreads[tn].load_from_file(tn)
            self.jobs_finished[tn] = []
            # force these to the front of the line
            new_thread_ordering.append(tn)

        self.thread_ordering = new_thread_ordering + self.thread_ordering
#        print "thread ordering now:", self.thread_ordering

        # now run a bit
        for crap in xrange(0, innercount):
            self.poll_queue_and_update_status(crap == 0)

            # identify ready to run and dependencies resolved jobs
            for t in iter(self.jobs_finished):
                self.jthreads[t].update(self.jobs_finished[t])
                self.jobs_finished[t] = []

            # submit jobs if possible
            self.submit_jobs()

            # snooze a bit
            time.sleep(25)  # in seconds

        if self.thread_bundle.is_done():
            print "ALL THREADS COMPLETED"

